package common;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

public class PropertiesReader {
	private static final String DEFAULT_FILENAME="application.properties";
	private static final long CHECK_WAIT=1000;
	private static PropertiesReader propertiesReader;
	private Properties properties;
	private File file;
	private long nextCheck;
	private long lastFileTime;
	private Logger logger;
	
	public static PropertiesReader getInstance() {
		if (propertiesReader==null) {
			try {
				propertiesReader = new PropertiesReader();
			} catch (IOException ioe) {
				throw new RuntimeException(ioe);
			}
		}
		return propertiesReader;
	}

	public PropertiesReader() throws IOException {
		this.file=new File(DEFAULT_FILENAME);
		if (!file.exists()) {
			File file=new File(new File(".."),DEFAULT_FILENAME);
			if (file.exists()) {
				this.file=file;
			}
		}
		load();
	}
	
	private void refresh() {
		if (System.currentTimeMillis() > nextCheck) {
			if (file.lastModified() != lastFileTime) {
				try {
					load();
					System.out.println("Loaded application.properties from " + file.getAbsolutePath());
				} catch (IOException e) {
					//logger.error("Failed to load from " + file.getAbsolutePath());
				}
			}
		}
	}
	
	private void load() throws IOException {
		properties=new Properties();
		if (file.exists()) {
			properties.load(new FileInputStream(file));
			nextCheck=System.currentTimeMillis() + CHECK_WAIT;
			lastFileTime=file.lastModified();
		} else {
			properties.put("util.Logger.logLevel", "INFO");
		}
	}
	public String getProperty(String key, boolean silent) {
		refresh();
		String value=(String)properties.get(key);
		if (value==null && !silent) {
			if (logger==null) {
				logger=Logger.getLogger(this.getClass());
			}
			logger.debug("Property value (" + key + ") is null");
		}
		return value;
	}
	public String getProperty(String key) {
		return getProperty(key, false);
	}
	
	public int getIntProperty(String key) {
		String stringValue=getProperty(key);
		return Integer.parseInt(stringValue);
	}
	
	public boolean getBooleanProperty(String key) {
		String stringValue=getProperty(key);
		return Boolean.valueOf(stringValue).booleanValue();
	}
	public int getIntProperty(String key, int defaultValue) {
		String value=getProperty(key, true);
		return value==null?defaultValue : Integer.parseInt(value);
	}
	public long getLongProperty(String key, long defaultValue) {
		String value=getProperty(key, true);
		return value==null?defaultValue : Long.parseLong(value);
	}
	public boolean getBooleanProperty(String key, boolean defaultValue) {
		String value=getProperty(key, true);
		return value==null?defaultValue : Boolean.valueOf(value).booleanValue();
	}
	public String getProperty(String key, String defaultValue) {
		String value=getProperty(key, true);
		return value==null?defaultValue : value;
	}
	/**
	 * Returns true if the property exists.  
	 * If it is set to "" it still returns true
	 * @param key
	 * @return
	 */
	public boolean isProperty(String key) {
		return getProperty(key) != null;
	}
}
