package common.bluray;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import common.DirectMemoryManager;
import common.StreamDescriptor;
import common.StreamIdentifier;

public class PlayList {
	private static final long INT_BITS=0xffffffff;
	private static final long CHAPTER_FLAG_MASK=0xffff000000000000l;
	private static final long CHAPTER_FLAG=       0x01000000000000l;
	private static final long CHAPTER_MASK=~CHAPTER_FLAG_MASK;
//	private static final String STREAM_HOME="/BDMV/STREAM/";
	private static StreamIdentifier streamIdentifier=new StreamIdentifier();
	private TreeMap<Long, String> chapters;
	private List<Branch> branchList;
	private long length;
	private int branchSize=1;
	
	public TreeMap<Long, String> getChapters() {
		return chapters;
	}
	public List<Branch> getBranchList() {
		return branchList;
	}
	public long getLength() {
		return length;
	}
	public int getBranchSize() {
		return branchSize;
	}
	public class Branch {
		private long timeIn;
		private long timeOut;
		private String fileNames[]=new String[1];
		private TreeMap<Integer, StreamDescriptor> streamDescriptorMap=new TreeMap<Integer, StreamDescriptor>();
		
		private void parse(ByteBuffer buffer) {
			short len=buffer.getShort();
			int bufferEnd=buffer.position() + len;
			fileNames[0]=getFileName(getNonUnicodeString(buffer,0xa));
			int flags=buffer.get() & 0xff;
			buffer.get();//Unknown
			timeIn=buffer.getInt() & INT_BITS;
			timeOut=buffer.getInt() & INT_BITS;
			buffer.getLong();
			buffer.getInt();
			if (flags >=0x10) {
				int branchCount=buffer.get() & 0xff;
				buffer.get();
				String fileName=fileNames[0];
				fileNames=new String[branchCount];
				fileNames[0]=fileName;
				for (int i=1;i<branchCount;i++) {
					fileNames[i]=getFileName(getNonUnicodeString(buffer,0xa));
				}
				branchSize=Math.max(branchSize, branchCount);
			}
			buffer.getInt();
			int videoStreams=buffer.get() & 0xff;
			int audioStreams=buffer.get() & 0xff;
			int subtitleStreams=buffer.get() & 0xff;
			int branchStreams=buffer.get() & 0xff;
			int subVideo=buffer.get() & 0xff;
			int subAudio=buffer.get() & 0xff;
			int pip=buffer.get() & 0xff;
			buffer.get();//Unknown;
			buffer.getInt();//Unknown;
			int allStreams=videoStreams+audioStreams+subtitleStreams+branchStreams+subVideo+subAudio+pip;
			
			for (int i=0;i<allStreams;i++) {
				byte sanity=buffer.get();
				if (sanity != 9) {
					System.out.println("Parse Problem");
				}
				byte majorType=buffer.get();
				short pid;
				int skip;
				if (majorType==3) {
					buffer.get();//unknown
					pid=buffer.getShort();
					skip=5;
				} else {
					pid=buffer.getShort();
					skip=6;
				}
				buffer.position(buffer.position()+skip);
				sanity=buffer.get();
				if (sanity != 5) {
					System.out.println("Parse Problem");
				}
				byte tsType=buffer.get();
				String language=getNonUnicodeString(buffer, 4);
				if ((pid & 0x1f00)>=0x1a00) {
					//Don't know what these are.  Just exit
					break;
				}
				if (language.equals("a")) {
					language=null;
				} else if (language.charAt(0)=='a' && language.length()==4) {
					language=language.substring(1);
				}
				//if (majorType==1) {
					StreamDescriptor sd=streamIdentifier.getTsStreamDescriptor(pid & 0x1fff, tsType);
					if (language!=null) {
						sd.setLanguage(language);
					}
					streamDescriptorMap.put(Integer.valueOf(sd.getStreamId()), sd);
				/*} else if (majorType==3) {
					short unknown1=buffer.getShort();//Unknown
					//Have seen unknown1 with values of 0x0100 & 0x0200
					if (unknown1>=(short)0x0100) {
						buffer.getInt();
					}
				}*/
			}
			//Sometimes the buffer would have an extra int that would cause things to blow up
			if (buffer.position() != bufferEnd) {
				buffer.position(bufferEnd);
			}
		}
		public long getTimeMillis() {
			return (timeOut-timeIn)/45;
		}
		public String toString() {
			return fileNames[0] + ":" + streamDescriptorMap;
		}
		private String getFileName(String fileName) {
			return fileName.substring(0, fileName.length() -4) + '.' + fileName.substring(fileName.length() -4);
		}
		public String getFileName() {
			return fileNames[0];
		}
		public String getFileName(int i) {
			if (i>=fileNames.length) {
				i=0;
			}
			return fileNames[i];
		}
		public Map<Integer,StreamDescriptor> getStreamDescriptorMap() {
			return streamDescriptorMap;
		}
	}
	private String getNonUnicodeString(final ByteBuffer buffer, final int limit) {
		StringBuffer sb=new StringBuffer(limit);
		
		boolean copy=true;
		for (int i=0;i<limit;i++) {
			byte b=buffer.get();
			copy &=(b!=0);
			if (copy) {
				sb.append((char)b);
			}
		}
		return sb.toString();
	}
	/*public List<Branch> getBranchList(String playListName) throws IOException {
		return getBranchList(new File(playListName));
	}*/
	public void parse(File playListFile) throws IOException {
		FileChannel in=new RandomAccessFile(playListFile, "r").getChannel();
		
		ByteBuffer buffer=DirectMemoryManager.allocateDirect((int)in.size());
		in.read(buffer);
		buffer.position(4*16);
		
		final int files=buffer.getShort();
		branchList=new ArrayList<Branch>(files);
		length=0;
		chapters=new TreeMap<Long, String>();
		
		buffer.getShort();//Unknown;
		for (int i=0;i<files;i++) {
			Branch branch=new Branch();
			branch.parse(buffer);
			length +=branch.getTimeMillis();
			branchList.add(branch);
		}
		// Get Chapters
		int len=buffer.getInt();
		int chapterCount=buffer.getShort();
		for (int i=0;i<chapterCount;i++) {
			long begin=buffer.getLong();
			if ((begin & CHAPTER_FLAG_MASK)==CHAPTER_FLAG) {
				chapters.put(begin & CHAPTER_MASK, null);
			}
			buffer.position(buffer.position() + 6);
		}
	}

}
