package common.ts;

import java.util.ArrayList;
import java.util.List;

import common.StreamDescriptor;

import demux.ts.TsInfo;
import demux.ts.TsRawPacketPart;

public class TableHandler {
	public static final int MODE_COPY=0;
	public static final int MODE_PARSE=1;
	public static final int MODE_FILTER=2;
	protected static final TsRawPacketPart[] DUMMY=new TsRawPacketPart[0];
	
	protected final TsRawPacketPart[] COPY=new TsRawPacketPart[1];
	
	private int mode=MODE_COPY;
	private ArrayList<TsTable> tsTableList=new ArrayList<TsTable>(10);
	private boolean reparse;
	private boolean finished;
	private ProgramAssociationTable pat=new ProgramAssociationTable(this);
	
	public TableHandler() {
		add(pat);
	}
	
	public TsTable getTable(int pid) {
		for (int i=0;i<tsTableList.size();i++) {
			if (tsTableList.get(i).getPid()==pid) {
				return tsTableList.get(i);
			}
		}
		return null;		
	}
	
	public void add(TsTable tsTable) {
		tsTableList.add(tsTable);
	}
	public boolean remove(TsTable tsTable) {
		return tsTableList.remove(tsTable);
	}
	public boolean isFinished() {
		return finished;
	}

	public int getMode() {
		return mode;
	}

	public void setMode(int mode) {
		this.finished=false;
		this.mode = mode;
	}
	public boolean isPcrPid(int pid) {
		for (int i=0;i<tsTableList.size();i++) {
			
			if (tsTableList.get(i) instanceof ProgramMapTable) {
				ProgramMapTable pmt=(ProgramMapTable)tsTableList.get(i);
				if (pid==pmt.getPcrPid()) {
					return true;
				}
			}
		}
		return false;
	}
	public ProgramInfo[] getProgramInfo() {
		return pat.getProgramInfo();
	}
	
	public void setSelectedProgramNumbers(short programNumbers[]) {
		for (int i=tsTableList.size()-1;i>=0;i--) {
			tsTableList.get(i).setSelectedProgramNumbers(programNumbers, this);
		}
		this.setMode(MODE_FILTER);
	}
	public void setStreamList(List<StreamDescriptor> list) {
		for (int i=tsTableList.size()-1;i>=0;i--) {
			tsTableList.get(i).setStreamList(list);
		}
	}

	public ProgramAssociationTable getPat() {
		return pat;
	}
	
	public TsTable[] getTables(Class c) {
		int count=0;
		for (int i=0;i<tsTableList.size();i++) {
			if (tsTableList.get(i).getClass().equals(c)) {
				count++;
			}
		}
		TsTable tsTableArray[]=new TsTable[count];
		
		for (int i=0,j=0;i<tsTableList.size();i++) {
			if (tsTableList.get(i).getClass().equals(c)) {
				tsTableArray[j++]=tsTableList.get(i);
			}
		}
		return tsTableArray;
	}

	/**
	 * @param part
	 * @return null if not a table PID
	 */
	public TsRawPacketPart[] append(TsRawPacketPart part) {
		TsInfo tsInfo=part.getTsInfo();
		TsTable tsTable=getTable(tsInfo.pid);
		if (tsTable == null) {
			return null;
		} else if (mode==MODE_COPY || (mode==MODE_PARSE && finished)) {
			COPY[0]=part;
			return COPY;
		}else {
			boolean complete=tsTable.append(part);
			if (complete) {
				if (mode==MODE_PARSE) {
					if (!tsTable.isFound()) {
						TsRawPacketPart[] out=tsTable.parse();
						if (!finished) {
							finished=true;
							for (int i=0;i<tsTableList.size();i++) {
								tsTable=tsTableList.get(i);
								if (tsTable.isRequired() && !tsTable.isFound()) {
									finished=false;
									break;
								}
							}
						}
						return out;
					} else {
						COPY[0]=part;
						return COPY;
					}
				} else {
					return tsTable.filter(reparse);
				}
			} else {
				return DUMMY;
			}
		}
	}
}
