package common.ts;

import java.io.FileInputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;

import common.DirectMemoryManager;

import demux.ts.TsRawPacketPart;

public class TerrestrialVirtualChannelTable extends TsTable {
	private short selectedProgramNumbers[];
	private ChannelInfo[] channelInfo;
	
	public TerrestrialVirtualChannelTable() {
		super(TsConstants.ATSC_TABLE_PID, TsConstants.TERRESTRIAL_VIRTUAL_CHANNEL_TABLE_ID);
	}
	public TsRawPacketPart[] parse() {
		ByteBuffer tmp=mergeIn(buffer);
		//TODO: Add real offset
		tmp.position(10);
		int channelCount=tmp.get() & 0xff;
		channelInfo=new ChannelInfo[channelCount];
		for (int i=0;i<channelCount;i++) {
			char shortName[]=new char[7];
			for (int j=0;j<shortName.length;j++) {
				shortName[j]=tmp.getChar();
			}
			channelInfo[i]=new ChannelInfo();
			channelInfo[i].setShortName(new String(shortName));
			int majorChannel=(tmp.get() & 0xf) << 6;
			int tmpChannel=tmp.get() & 0xff;
			majorChannel+=tmpChannel >>> 2;
			channelInfo[i].setMajorChannel(majorChannel);
			channelInfo[i].setMinorChannel(((tmpChannel & 0x3) << 8) + (tmp.get() & 0xff));
			tmp.get();//Modulation Mode
			tmp.getInt();//Carrier Frequency
			tmp.getShort();//Channel TSID;
			channelInfo[i].setProgramNumber(tmp.getShort());
			tmp.getShort();//Stuff
			tmp.getShort();//sourceId
			int descLen=tmp.getShort() & 0x3ff;
			tmp.position(tmp.position() + descLen);
			
		}
		found=true;
		notifyListeners();
		return buffer;
	}
	
	public TsRawPacketPart[] filter(boolean reparse) {
		if (out==null || reparse) {
			ByteBuffer tmp=mergeIn(buffer);
			
			out=DirectMemoryManager.allocateDirect(getLen(buffer));
			copy(tmp,out,10);
			out.put((byte)selectedProgramNumbers.length);
			int channelCount=tmp.get() & 0xff;
			byte stuffBuffer[]=new byte[24];
			for (int i=0;i<channelCount;i++) {
				tmp.get(stuffBuffer);
				short programNumber=tmp.getShort();
				int stuff=tmp.getInt();
				short descLen=tmp.getShort();
				byte descBuffer[]=new byte[descLen & 0x3ff];
				tmp.get(descBuffer);
				for (int j=0;i<selectedProgramNumbers.length;j++) {
					if (programNumber==selectedProgramNumbers[j]) {
						out.put(stuffBuffer);
						out.putShort(programNumber);
						out.putInt(stuff);
						out.putShort(descLen);
						out.put(descBuffer);
						break;
					}
				}
			}
			short descLen=tmp.getShort();
			out.putShort(descLen);
			copy(tmp,out,descLen & 0x3ff);
			finalizeOut();
		}
		return mergeOut(buffer);
	}
	public static void main(String args[]) {
		try {
			FileChannel in=new FileInputStream("M:/tools/My Name Is Earl-(Stole a Motorcycle)-2008-04-10-0.ts").getChannel();
			ByteBuffer buffer=in.map(FileChannel.MapMode.READ_ONLY,3*188,188);
			TerrestrialVirtualChannelTable tvct=new TerrestrialVirtualChannelTable();
			
			tvct.append(new TsRawPacketPart(buffer));
			tvct.parse();
			ChannelInfo info[]=tvct.getChannelInfo();
			short programNumbers[]=new short[info.length];
			for (int i=0;i<info.length;i++) {
				System.out.println(info[i]);
				programNumbers[i]=info[i].getProgramNumber();
			}
			/*TsRawPacketPart out[]=tvct.filter(programNumbers, false);
			ByteBuffer test=out[0].getByteBuffer();
			for (int i=0;i<buffer.capacity();i++) {
				if (buffer.get(i) != test.get(i)) {
					System.out.println("Failed at: " + i + " - " + Integer.toString(buffer.get(i) & 0xff, 16) + ':' + Integer.toString(test.get(i) & 0xff, 16));
				}
			}*/
			//System.out.println(pmt.getProgramNumber());
			//pmt.filter(new HashMap(), true);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	public void setSelectedProgramNumbers(short[] programNumbers) {
		this.selectedProgramNumbers = programNumbers;
	}
	public ChannelInfo[] getChannelInfo() {
		return channelInfo;
	}
	protected void tsTableDidParse(TsTable tsTable, TableHandler tableHandler) {
		//Set our pid based on MasterGuideTable
		if (tsTable instanceof MasterGuideTable) {
			MasterGuideTable mgt=(MasterGuideTable)tsTable;
			TableInfo tableInfo[]=mgt.getTableInfo();
			for (int i=0;i<tableInfo.length;i++) {
				if (tableInfo[i].getType()==MasterGuideTable.TVCT_CURRENT) {
					pid=tableInfo[i].getPid();
					break;
				}
			}
		}
	}
}
