package io;

import java.io.IOException;
import java.io.InputStream;

public class MonitoredInputStream extends InputStream {
	private InputStream in;
	private long bytes;
	private InputStreamMonitorI inputStreamMonitor;
	
	public MonitoredInputStream(InputStream in, InputStreamMonitorI inputStreamMonitor) {
		this.in=in;
		this.inputStreamMonitor=inputStreamMonitor;
	}

	public int read() throws IOException {
		int myByte=in.read();
		if (myByte >= 0) {
			bytes++;
		}
		return myByte;
	}
	public int read(byte buffer[]) throws IOException {
		return read(buffer, 0, buffer.length);
	}
	public int read(byte buffer[], int offset, int len) throws IOException {
		int myBytes=in.read(buffer, offset, len);
		if (myBytes > 0) {
			bytes+=myBytes;
			this.inputStreamMonitor.bytesAccessed(bytes);
		}
		return myBytes;
	}
	
	public void close() throws IOException {
		in.close();
		inputStreamMonitor.streamClosed();
	}
	public int available() throws IOException {
		return in.available();
	}
	public long skip(long n) throws IOException {
		long bytesSkipped=in.skip(n);
		this.bytes+=bytesSkipped;
		this.inputStreamMonitor.bytesAccessed(bytes);
		return bytesSkipped;
	}
	public boolean markSupported() {
		return in.markSupported();
	}
	public void mark(int readlimit) {
		in.mark(readlimit);
	}
	public void reset() throws IOException {
		in.reset();
	}
}
