package muxer;

import java.io.IOException;

import common.packet.Packet;


public abstract class MuxFilter implements Comparable {
	protected static int POSITION_POST_DEMUX=10;
	protected static int POSITION_PRE_BUFFER=20;
	protected static int POSITION_BUFFER=30;
	protected static int POSITION_POST_BUFFER=40;
	protected static int POSITION_PRE_MUXER=50;
	protected static int POSITION_MUXER=60;
	protected static int POSITION_POST_MUXER=70;
	
	protected MuxManager muxManager;
	protected MuxFilter nextFilter;
	
	/**
	 * @return returns the container types that this interface has RAW support for
	 */
	public boolean handlesRaw(int container) {
		return false;
	}
	
	public int addPacket(Packet packet) throws IOException {
		if (nextFilter==null) {
			return MuxManager.CONTINUE;
		} else {
			return nextFilter.addPacket(packet);
		}
	}
	
	public void streamEnd() throws IOException {
		if (nextFilter!=null) {
			nextFilter.streamEnd();
		}
	}

	public void setNextFilter(MuxFilter nextFilter) {
		this.nextFilter = nextFilter;
	}
	public boolean requiresBuffer() {
		return false;
	}

	public MuxManager getMuxManager() {
		return muxManager;
	}

	public void setMuxManager(MuxManager muxManager) {
		this.muxManager = muxManager;
	}
	
	protected void init() throws IOException {};
	
	public abstract int getPosition();
	
	public int compareTo(Object o) {
		if (!(o instanceof MuxFilter)) {
			return 1;
		}
		MuxFilter muxFilter=(MuxFilter)o;
		return getPosition() - muxFilter.getPosition();
	}
}
