package muxer.filter;

import java.io.IOException;
import java.util.Map;
import java.util.TreeMap;

import muxer.MuxFilter;
import muxer.MuxManager;

import common.StreamDescriptor;
import common.StreamIdentifier;
import common.mpeg.MpegConstants;
import common.packet.Packet;

public class PesStreamIdentifierMuxFilter extends MuxFilter {
	private static final int SCAN_SIZE=200*1024*1024;
	private StreamIdentifier streamIdentifier=new StreamIdentifier();
	
	private int bytes;
	private TreeMap<Integer, StreamDescriptor> streamDescriptorMap=new TreeMap<Integer, StreamDescriptor>();
	
	public int addPacket(Packet packet) throws IOException {
		bytes += packet.getPayload().limit();
		int streamId=packet.getStreamId();
		StreamDescriptor streamDescriptor=(StreamDescriptor)streamDescriptorMap.get(Integer.valueOf(streamId));
		if (streamDescriptor==null || streamDescriptor.getClass()==StreamDescriptor.class) {
			streamDescriptor=streamIdentifier.identifyStream(streamId, packet.getPayload());
			if (streamDescriptor==null) {
				if ((streamId & 0xF0)==0x20) {
					streamDescriptor=new StreamDescriptor(streamId, StreamDescriptor.CLASSIFICATION_SUBTITLE);
					streamDescriptor.setCodec("Subtitle");
					streamDescriptor.setTsType(MpegConstants.TS_STREAM_TYPE_SUBTITLE);
				}
			} 
			if (streamDescriptor!=null) {
				streamDescriptorMap.put(Integer.valueOf(streamId), streamDescriptor);
			}
		}
		//System.out.println("Bytes: " + bytes/1024/1024);
		if (bytes < SCAN_SIZE) {
			return MuxManager.CONTINUE;
		} else {
			return MuxManager.ABORT;
		}
	}

	public Map<Integer, StreamDescriptor> getStreamDescriptorMap() {
		return streamDescriptorMap;
	}
	public int getPosition() {
		return POSITION_POST_BUFFER;
	}	
}
