package muxer.ts;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.WritableByteChannel;
import java.util.Iterator;
import java.util.LinkedList;

import common.mpeg.MpegConstants;
import common.packet.ClockReferenceI;

import demux.ts.M2tsRawPacketPart;
import demux.ts.TsInfo;
import demux.ts.TsRawPacketPart;

/**
 * WARNING!  WARNING!  Caches PacketParts
 *
 */
public class M2tsPacketWriter extends TsPacketWriter {
	private LinkedList<M2tsRawPacketPart> packetList=new LinkedList<M2tsRawPacketPart>();
	//private Stack packetRecycler=new Stack();
	private int syncStreamId=Integer.MIN_VALUE;
	private int syncCount=0;
	private long offset=MpegConstants.PTS_FREQ;//1 second
	
	//Used for end of stream
	private long timePerByte;
	private long arrivalTime;
	
	public M2tsPacketWriter(WritableByteChannel out) {
		super(out);
	}
	public void writeTsPacket(TsRawPacketPart packetPart) throws IOException {
		if (packetPart.getClass() == TsRawPacketPart.class ) {
			packetPart=new M2tsRawPacketPart(packetPart.getByteBuffer(),0);
		}
		TsInfo tsInfo=packetPart.getTsInfo();
		if (tsInfo.pid==syncStreamId && tsInfo.pidStart) {
			long pts=packetPart.getClockReference(ClockReferenceI.CLOCK_REFERENCE_TYPE_PTS);
			if (pts > 0) {
				syncCount++;
				if (syncCount==2) {
					calculateArrivalTime(pts);
					flush();
				}
			}
		}
		packetList.add((M2tsRawPacketPart)packetPart);
	}
	
	/**
	 * Calculates the arrival time and store the time/byte value as well
	 * @param endPts
	 */
	private void calculateArrivalTime(long endPts) {
		long preSyncBytes=0;
		long postSyncBytes=0;
		boolean foundSync=false;
		long startPts=Long.MIN_VALUE;
		for (M2tsRawPacketPart packetPart : packetList) {
			TsInfo tsInfo=packetPart.getTsInfo();
			if (!foundSync && tsInfo.pid==syncStreamId && tsInfo.pidStart) {
				startPts=packetPart.getClockReference(ClockReferenceI.CLOCK_REFERENCE_TYPE_PTS);
				foundSync=startPts>0;
			}
			if (foundSync) {
				postSyncBytes+=MpegConstants.M2TS_PACKET_SIZE;
			} else {
				preSyncBytes+=MpegConstants.M2TS_PACKET_SIZE;
			}
		}
		long arrivalDelta=(endPts-startPts)*300;
		
		//Cache these values
		timePerByte=arrivalDelta/postSyncBytes;
		arrivalTime=startPts*300 - timePerByte*preSyncBytes + offset*300;
		
	}
	
	private void flush() throws IOException {
		for (Iterator<M2tsRawPacketPart> i=packetList.iterator();i.hasNext();) {
			M2tsRawPacketPart packetPart=i.next();
			
			packetPart.setArrivalTime((int)arrivalTime & M2tsRawPacketPart.ARRIVAL_TIME_MASK);
			packetPart.write(out);
			arrivalTime += timePerByte * MpegConstants.M2TS_PACKET_SIZE;
			i.remove();
			//packetRecycler.push(packetPart);
		}
		syncCount--;
	}
	
	public TsRawPacketPart getPacketPart() {
		return new M2tsRawPacketPart(188, 0);
	}	
	public void streamEnd() throws IOException {
		flush();
		super.streamEnd();
	}
	public void setSyncStreamId(int syncStreamId) {
		this.syncStreamId = syncStreamId;
	};
	
}
