package ui;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.List;

import common.Util;

public class GUIDToHex extends Frame {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		new GUIDToHex().setVisible(true);
	}
	
	private TextField guid;
	
	private TextField hex;
	
	public GUIDToHex() {
		this.setTitle("GUID to Hex");
		this.setLayout(new BorderLayout());
		Panel west=new Panel(new GridLayout(2,1));
		west.add(new Label("GUID:"));
		west.add(new Label("Hex:"));
		this.add(west, BorderLayout.WEST);
		
		Panel center=new Panel(new GridLayout(2,1));
		center.add(guid=new TextField());
		Dimension dim=guid.getPreferredSize();
		dim.width=300;
		guid.setPreferredSize(dim);
		center.add(hex=new TextField());
		this.add(center, BorderLayout.CENTER);
		
		Panel right=new Panel(new GridLayout(2,1));
		Button toHex=new Button("To Hex");
		Button toGuid=new Button("To Guid");
		right.add(toHex);
		right.add(toGuid);
		this.add(right, BorderLayout.EAST);
		this.pack();
		toHex.addActionListener( new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				toHex();
			}
		});
		
		toGuid.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				toGuid();
			}
		});
		
		this.addWindowListener(new WindowAdapter(){
			@Override
			public void windowClosing(WindowEvent e) {
				dispose();
			}
		});
	}
	
	private void toHex() {
		String guidText=guid.getText().trim().toLowerCase();
		ByteBuffer buffer=ByteBuffer.allocate(16);
		buffer.order(ByteOrder.LITTLE_ENDIAN);
		int pointer=0;
		long l=Long.parseLong(guidText.substring(pointer, pointer + 8), 16);
		buffer.putInt((int)l);
		pointer +=9;
		l=Long.parseLong(guidText.substring(pointer, pointer + 4), 16);
		buffer.putShort((short)l);
		pointer +=5;
		l=Long.parseLong(guidText.substring(pointer, pointer + 4), 16);
		buffer.putShort((short)l);
		pointer +=5;
		
		buffer.order(ByteOrder.BIG_ENDIAN);
		l=Long.parseLong(guidText.substring(pointer, pointer + 4), 16);
		buffer.putShort((short)l);
		pointer +=5;
		
		while (pointer < guidText.length()) {
			l=Long.parseLong(guidText.substring(pointer, pointer + 2), 16);
			buffer.put((byte)l);
			pointer+=2;
		}
		
		StringBuffer sb=new StringBuffer(16*3);
		for (int i=0;i<buffer.position();i++) {
			Util.appendHex(buffer.get(i), sb);
			sb.append(' ');
		}
		hex.setText(sb.toString().trim());
	}
   //2C 80 6D E0 46 DB CF 11 B4 D1 00 80 5F 6C BB EA = e06d802c-db46-11cf-b4d1-00805f6cbbea
	private void toGuid() {
		byte bytes[]=Util.hexToBytes(hex.getText().trim());
		guid.setText(toGuid(bytes));
	}
	
	public static String toGuid(byte[] bytes) {
		ByteBuffer buffer=ByteBuffer.wrap(bytes);
		buffer.order(ByteOrder.LITTLE_ENDIAN);
		int i=buffer.getInt();
		StringBuffer sb=new StringBuffer(40);
		Util.longToHex(i & 0xffffffffl, 8, sb);
		sb.append('-');
		int s=buffer.getShort();
		Util.longToHex(s & 0xffffl, 4, sb);
		sb.append('-');
		s=buffer.getShort();
		Util.longToHex(s & 0xffffl, 4, sb);
		sb.append('-');
		buffer.order(ByteOrder.BIG_ENDIAN);
		s=buffer.getShort();
		Util.longToHex(s & 0xffffl, 4, sb);
		sb.append('-');
		
		while (buffer.remaining() > 0) {
			Util.appendHex(buffer.get(), sb);
		}
		return sb.toString().toLowerCase();
	}
}
