package util;

import java.io.File;
import java.io.IOException;
import java.util.List;

import muxer.MuxFilter;
import muxer.MuxManager;
import muxer.filter.AvcSequenceEndMuxFilter;
import muxer.filter.ContiguousTimestampMuxFilter;
import muxer.filter.CoreMuxFilter;
import muxer.ts.M2tsPesMuxer;
import muxer.ts.M2tsRawMuxer;
import muxer.ts.TsPesMuxer;
import muxer.ts.TsRawMuxer;
import ui.tonmt.ToNMTPrefs;

import common.AudioStreamDescriptor;
import common.BufferedWritableByteChannel;
import common.FileStream;
import common.Job;
import common.StreamDescriptor;
import common.StreamIdentifier.LpcmAnalyzer;
import common.avc.AvcConstants;
import common.mpeg.MpegConstants;

import demux.Demuxer;
import demux.ts.M2tsRawDemuxer;
import demux.ts.TsRawDemuxer;

public class InternalStreamConverter extends StreamConverter {
	private boolean failOnOnlyLPCM=true;
	
	public InternalStreamConverter(ToNMTPrefs prefs) {
		Demuxer demuxer=new M2tsRawDemuxer();
		muxManager=new MuxManager(demuxer);
		this.prefs=prefs;
	}	
	public void convert(FileStream fileStream, String outputFileName, List<StreamDescriptor> list) throws IOException {
		Demuxer demuxer=muxManager.getDemuxer();
		demuxer.setStreams(list);
		
		logger.info("Remuxing:");
		StreamDescriptor video=getSubList(list, StreamDescriptor.CLASSIFICATION_VIDEO).get(0);
		
		List<StreamDescriptor> audioList=getSubList(list, StreamDescriptor.CLASSIFICATION_AUDIO);
		if (failOnOnlyLPCM) {
			int lpcmCount=0;
			for (StreamDescriptor audio : audioList) {
				if (LpcmAnalyzer.CODEC_NAME.equals(audio.getCodec())) {
					lpcmCount++;
				}
			}
			if (lpcmCount==audioList.size()) {
				logger.error("LPCM requires another audio stream to play");
				fileStream.getJob().setStatus(Job.STATUS_FAILED);
				return;
			}
		}
		if (prefs.isCore()) {
			for (StreamDescriptor audio : audioList) {
				AudioStreamDescriptor asd=(AudioStreamDescriptor)audio;
				StreamDescriptor coreSd=asd.getCoreStreamDescriptor();
				if (coreSd!=null) {
					muxManager.add(new CoreMuxFilter(asd.getStreamId(), coreSd.getStreamId()));
					//Change the type of the stream to the type of the core
					asd.setTsType(coreSd.getTsType());
				}
			}
		}
		if (fileStream.getFiles().length > 1) {
			if (AvcConstants.CODEC_NAME.equals(video.getCodec())) {
				muxManager.add(new AvcSequenceEndMuxFilter(video.getStreamId()));
			}
			muxManager.add(new ContiguousTimestampMuxFilter());
		}
		//WritableByteChannel out=new FileOutputStream(outputFileName).getChannel();
		MuxFilter muxer;
		if (outputFileName.toUpperCase().endsWith(BLURAY_EXTENSION)) {
			BufferedWritableByteChannel out=new BufferedWritableByteChannel(new File(outputFileName),MpegConstants.M2TS_PACKET_SIZE*50000);
			if (demuxer instanceof TsRawDemuxer) {
				muxer=new M2tsRawMuxer(out);
			} else {
				muxer=new M2tsPesMuxer(out);
			}
		} else {
			BufferedWritableByteChannel out=new BufferedWritableByteChannel(new File(outputFileName),MpegConstants.TS_PACKET_SIZE*50000);
			if (demuxer instanceof TsRawDemuxer) {
				muxer=new TsRawMuxer(out);
			} else {
				muxer=new TsPesMuxer(out);
			}
		}
		muxManager.add(muxer);
		muxManager.mux(fileStream);
		//fileStream.close();
	}
	
	//TODO: Implement this
	public void abort() {}
}
