package util;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.channels.FileChannel;
import java.util.Arrays;

import common.Util;

public class WtvAnalyzer {

	public static final byte DATA_GUID[]=hexToBytes("95 C3 D2 C2 7E 9A DA 11 8B F7 00 07 E9 5E AD 8D");
	public static final byte TIMING_GUID[]=hexToBytes("5B 05 E6 1B 97 A9 49 43 88 17 1A 65 5A 29 8A 97");
	public static final byte STREAM_DATA_GUID[]=hexToBytes("ED A4 13 23 2D BF 4F 45 AD 8A D9 5B A7 F9 1F EE");
	public static final byte LANGUAGE_GUID[]=hexToBytes("6D 66 92 E2 02 9C 8D 44 AA 8D 78 1A 93 FD C3 95");
	public static final byte STEAM_VIDEO[]=hexToBytes("76 69 64 73");
	public static final byte STEAM_AUDIO[]=hexToBytes("61 75 64 73");
	
	private static final byte[] META_GUID=hexToBytes("5A FE D7 6D C8 1D 8F 4A 99 22 FA B1 1C 38 14 53");
	
	private static final byte[] TEST_GUID=hexToBytes("A2 C3 D2 C2 7E 9A DA 11 8B F7 00 07 E9 5E AD 8D");
	
	private static final byte STRING_META=0x01;
	private static final byte INTEGER_META=0x02;
	private static final byte BOOLEAN_META=0x03;
	private static final byte CLASS_META=0x06;
	
	private static byte[] hexToBytes(String hex) {
		hex=hex.toLowerCase();
		int byteCount=(hex.length() + 1)/3;
		byte bytes[]=new byte[byteCount];
		for (int i=0;i<byteCount;i++) {
			bytes[i]=(byte)Integer.parseInt(hex.substring(i * 3, i * 3 + 2), 16);
		}
		return bytes;
	}
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			//String fileName="C:/movies/Chain Reaction_KTVDDT_2010_02_11_18_58_00.wtv";
			//String fileName="D:/Video/American Idol_KDVRDT_2010_04_20_18_58_00.wtv";
			//String fileName="D:/Video/Friends_WUHF_2010_05_28_12_29_00.wtv";
			//String fileName="D:/Video/german_france_2lang_sd_stereo.wtv";
			//String fileName="D:/Video/german_sd_ac3.wtv";
			
			String fileName="D:/Video/The_Simpsons_TV3_2010_05_29_18_58_00.wtv";
			FileChannel in=new FileInputStream(new File(fileName)).getChannel();
			//FileChannel mpvOut=new FileOutputStream(new File("C:/movies/chain.mpv")).getChannel();
			//FileChannel ac3Out=new FileOutputStream(new File("C:/movies/chain.ac3")).getChannel();
			
			ByteBuffer buffer=in.map(FileChannel.MapMode.READ_ONLY, 0x40000, Math.min(100*1024*1024,in.size() - 0x40000));
			buffer.order(ByteOrder.LITTLE_ENDIAN);
			byte guid[]=new byte[16];
			while (true) {
				int pos=buffer.position();
				System.out.print(Integer.toString(pos + 0x40000, 16) + "-");
				buffer.get(guid);
				Util.printHex(guid, 0, guid.length);
				int len=buffer.getInt();
				
				byte bytes[]=new byte[4*48];
				System.out.print("(" + Integer.toString(len, 16) + ") ");
				int toGet=Math.min(bytes.length, len - 20);
				buffer.get(bytes, 0, toGet);
				Util.printHex(bytes, 0, toGet);
				if (Arrays.equals(guid, DATA_GUID)) {
					if (bytes[0]==0x13) {
						int i=0;
					}
				}
				if (Arrays.equals(guid, LANGUAGE_GUID)) {
					int i=0;
				}
				if (Arrays.equals(guid, STREAM_DATA_GUID)) {
					for (int i=0; i<4; i++) {
						System.out.print((char)buffer.get(pos+16+4+0x28 + i));
					}
				}
				if (Arrays.equals(guid, TEST_GUID)) {
					int i=0;
				}
				System.out.println();
				/*if (Arrays.equals(guid, DATA_GUID)) {
					int stream=buffer.getInt();
					long seq=buffer.getLong();
					buffer.limit(pos+len);
					if (stream==0x12) {
						ac3Out.write(buffer);
					} else if (stream==0x13) {
						mpvOut.write(buffer);
					}
					buffer.limit(buffer.capacity());
				}*/
				//qword align
				if (len % 8 != 0) {
					len += 8- (len % 8);
				}
				buffer.position(pos + len);
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
	private static void dumpMeta(ByteBuffer buffer) throws IOException {
		
	}
}
