package web;

import java.io.IOException;
import java.io.InputStream;
import java.net.InetAddress;
import java.net.URLDecoder;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import common.Logger;

public class MinimalServletRequest {
	private static final int MAX_LINE_SIZE=512;
	public static final String UTF8="UTF-8";
	private static final String COOKIE="COOKIE";
	public static final String AUTHORIZATION="Authorization";
	public static final String HOST="Host";
	public static final String RANGE="Range";
	public static final String WAP_PROFILE="x-wap-profile";
	public static final String EXPECT="Expect";
	
	private InputStream inputStream;
	private Map headers = new HashMap();
	private Map parameters = new HashMap();
	private List cookies=new ArrayList();
	private String method;
	private String protocol;
	private String path;
	private String remoteAddr;
	private Logger logger=Logger.getLogger(this.getClass());
	
	public MinimalServletRequest(InputStream inputStream, InetAddress remoteInetAddress) {
		this.inputStream=inputStream;
		remoteAddr=remoteInetAddress.getHostAddress();
	}
	
	public String getRequestURI() {
		return null;
	}
	public final String readLine(InputStream in) throws IOException {
		StringBuffer buffer=new StringBuffer();
		for (int myByte; (myByte=in.read()) != 10 && buffer.length() <= MAX_LINE_SIZE;) {
			if (myByte <0) {
				throw new IOException("Stream closed");
			}
			if (myByte !=13) {
				buffer.append((char)myByte);
			}
		}
		return buffer.toString();
	}
	public void parse() throws IOException {
		String request=readLine(inputStream);
		if (request.length() >= MAX_LINE_SIZE) {
			logger.info("Line too long: " + request);
			throw new IOException("Line too long");
		}
		method=request.substring(0, request.indexOf(' '));
		protocol=request.substring(request.lastIndexOf(' ')+1);
		path=request.substring(method.length(), request.length()-protocol.length()).trim();
		logger.debug("Request: " + request);
		
		boolean logHttpHeaders=logger.isDebug();
		for (String line;(line=readLine(inputStream)) != null && line.length() > 0;) {
			if (logHttpHeaders) {
				logger.trace(line);
			}
			String key=line.substring(0, line.indexOf(':')).toUpperCase();
			String value=line.substring(key.length() + 1).trim();
			if (key.equalsIgnoreCase(COOKIE)) {
				//Break the cookie into pieces.  :)
				String pieces[]=value.split("; ");
				String data[]=pieces[0].split("=");
				cookies.add(new Cookie(data[0],data[1]));
				//TODO: Add all the other parts
			} else {
				headers.put(key, value);
			}
		}
		int paramPos=path.indexOf('?');
		if (paramPos >=0 ) {
			String encodeParameters=path.substring(paramPos + 1);
			String encodedParameterArray[] = encodeParameters.split("&");
			for (int i=0;i<encodedParameterArray.length;i++) {
				String parameterArray[]=encodedParameterArray[i].split("=");
				//Deal with null
				if (parameterArray.length==2) {
					String key=URLDecoder.decode(parameterArray[0], UTF8);
					String value=URLDecoder.decode(parameterArray[1], UTF8);
					parameters.put(key, value);
				}
			}
		}
	}
	
	public String getPath() {
		return path;
	}
	public String getHeader(String name) {
		return (String)headers.get(name.toUpperCase());
	}
	public String getParameter(String key) {
		return (String)parameters.get(key);
	}
	public String getContentType() {
		return getHeader(MinimalServletResponse.CONTENT_TYPE);
	}
	public String getRemoteAddr() {
		return this.remoteAddr;
	}
	public InputStream getInputStream() {
		return inputStream;
	}
	public Cookie[] getCookies() {
		Cookie cookies[]=new Cookie[this.cookies.size()];
		this.cookies.toArray(cookies);
		return cookies;
	}
	public int getContentLength() {
		String value=getHeader(MinimalServletResponse.CONTENT_LENGTH);
		if (value == null) {
			return -1;
		}
		return Integer.parseInt(value);
	}
	public String getMethod() {
		return method;
	}
	public String getProtocol() {
		return protocol;
	}
	public Enumeration getHeaderNames() {
		return new Vector(headers.keySet()).elements();
	}
}
